from pathlib import Path
from typing import Any, Optional, cast

import pytest
import yaml

from pytest import MonkeyPatch

from hive.common import langchain as langchain_module
from hive.common.langchain import init_chat_model
from hive.common.testing import test_config_dir  # noqa: F401


def test_no_provider() -> None:
    assert _test_init_chat_model("gpt-4.1-nano") == dict(
        model="gpt-4.1-nano",
        model_provider=None,
    )


def test_non_ollama_provider() -> None:
    assert _test_init_chat_model(
        "gpt-4.1-nano",
        model_provider="openai",
    ) == dict(
        model="gpt-4.1-nano",
        model_provider="openai",
    )


def test_ollama_provider() -> None:
    assert _test_init_chat_model(
        "qwen3:0.6b",
        model_provider="ollama",
    ) == dict(
        model="qwen3:0.6b",
        model_provider="ollama",
        base_url="https://gbenson.net/ollama",
        client_kwargs={"auth": ("hello", "world")},
    )


def test_non_ollama_provider_in_model() -> None:
    assert _test_init_chat_model("openai:gpt-4.1-nano") == dict(
        model="openai:gpt-4.1-nano",
        model_provider=None,
    )


def test_ollama_provider_in_model() -> None:
    assert _test_init_chat_model("ollama:qwen3:0.6b") == dict(
        model="ollama:qwen3:0.6b",
        model_provider=None,
        base_url="https://gbenson.net/ollama",
        client_kwargs={"auth": ("hello", "world")},
    )


NON_OLLAMA_VARIATIONS = (
    dict(model="gpt-4.1-nano"),
    dict(model="gpt-4.1-nano", model_provider="openai"),
    dict(model="openai:gpt-4.1-nano"),
)

OLLAMA_VARIATIONS = (
    dict(model="qwen3:0.6b", model_provider="ollama"),
    dict(model="ollama:qwen3:0.6b"),
)

VARIATIONS = NON_OLLAMA_VARIATIONS + OLLAMA_VARIATIONS


@pytest.mark.parametrize("kwargs", VARIATIONS)
def test_no_config(kwargs: dict[str, Any], config_path: Path) -> None:
    """Nothing changes if we don't have the config file.
    """
    config_path.unlink()
    expect_kwargs = {"model_provider": None, **kwargs}
    assert _test_init_chat_model(**kwargs) == expect_kwargs


@pytest.mark.parametrize("kwargs", NON_OLLAMA_VARIATIONS)
def test_no_auth_wrong_provider(kwargs: dict[str, Any]) -> None:
    """Auth isn't added to non-ollama models even with our base_url.
    """
    kwargs = {"base_url": "https://gbenson.net/ollama", **kwargs}
    expect_kwargs = {"model_provider": None, **kwargs}
    assert _test_init_chat_model(**kwargs) == expect_kwargs


@pytest.mark.parametrize("kwargs", VARIATIONS)
def test_no_auth_wrong_base_url(kwargs: dict[str, Any]) -> None:
    """Auth isn't added if the base_url isn't ours.
    """
    kwargs = {"base_url": "https://gbenson.net/pyjama", **kwargs}
    expect_kwargs = {"model_provider": None, **kwargs}
    assert _test_init_chat_model(**kwargs) == expect_kwargs


@pytest.mark.parametrize(
    "with_base_url",
    (None,
     "https://gbenson.net/ollama",
     "https://gbenson.net/nollama",
     ))
@pytest.mark.parametrize("kwargs", VARIATIONS)
def test_no_overwrite_auth(
        with_base_url: Optional[str],
        kwargs: dict[str, Any],
) -> None:
    """Provided auth is never replaced.
    """
    kwargs = {"client_kwargs": {"auth": None}, **kwargs}
    if with_base_url:
        kwargs["base_url"] = with_base_url

    expect_kwargs = {"model_provider": None, **kwargs}
    if not with_base_url and "'ollama" in str(kwargs):
        expect_kwargs["base_url"] = "https://gbenson.net/ollama"

    assert _test_init_chat_model(**kwargs) == expect_kwargs


# Fixtures etc

@pytest.fixture
def config_path(test_config_dir: str | Path) -> Path:  # noqa: F811
    return Path(test_config_dir) / "ollama.yml"


@pytest.fixture(autouse=True)
def common_setup(config_path: Path, monkeypatch: MonkeyPatch) -> None:
    config_path.write_text(yaml.dump({
        "ollama": {
            "url": "https://gbenson.net/ollama",
            "http_auth": {
                "username": "hello",
                "password": "world",
            },
        },
    }))
    monkeypatch.setattr(
        langchain_module,
        "_init_chat_model",
        lambda model, **kwargs: {"model": model, **kwargs},
    )


def _test_init_chat_model(*args: Any, **kwargs: Any) -> dict[str, Any]:
    return cast(dict[str, Any], init_chat_model(*args, **kwargs))



from datetime import timedelta

import pytest

from hive.common import units


def test_names():
    assert {attr for attr in dir(units) if attr[0].isupper()} == {
        "BYTE", "KiB", "MiB", "GiB", "TiB",
        "SECOND", "MINUTE", "HOUR", "DAY",
        "MILLISECOND", "MICROSECOND",
    }


@pytest.mark.parametrize(
    "attr,expect_value",
    (("BYTE", 1),
     ("GiB", 1_073_741_824),
     ("KiB", 1024),
     ("MiB", 1_048_576),
     ("TiB", 1099511627776),
     ))
def test_integers(attr, expect_value):
    value = getattr(units, attr)
    assert isinstance(value, int)
    assert value == expect_value


@pytest.mark.parametrize(
    "attr,expect_value",
    (("DAY", 86400),
     ("HOUR", 3600),
     ("MICROSECOND", 1e-6),
     ("MILLISECOND", 0.001),
     ("MINUTE", 60),
     ("SECOND", 1),
     ))
def test_timedeltas(attr, expect_value):
    value = getattr(units, attr)
    assert isinstance(value, timedelta)
    assert value.total_seconds() == expect_value



from functools import wraps
from typing import Any, Optional

from langchain.chat_models import init_chat_model as _init_chat_model
from pydantic import BaseModel

from .config import read as read_config


@wraps(_init_chat_model)
def init_chat_model(
        model: str,
        *,
        model_provider: Optional[str] = None,
        **kwargs: Any
) -> Any:
    if _is_ollama_model(model, model_provider):
        if (config := _read_config("ollama")):
            kwargs = _configure_ollama_model(config, **kwargs)
    return _init_chat_model(model, model_provider=model_provider, **kwargs)


def _is_ollama_model(model: str, model_provider: Optional[str]) -> bool:
    return (model_provider == "ollama"
            if model_provider
            else model.startswith("ollama:"))


class Auth(BaseModel):
    username: str
    password: str

    def _as_tuple(self) -> tuple[str, str]:
        return self.username, self.password


class EndpointConfig(BaseModel):
    url: Optional[str] = None
    http_auth: Optional[Auth] = None


def _read_config(config_key: str) -> Optional[EndpointConfig]:
    try:
        config_dict = read_config(config_key)[config_key]
    except KeyError:
        return None
    return EndpointConfig.model_validate(config_dict)


def _configure_ollama_model(
        config: EndpointConfig,
        *,
        base_url: Optional[str] = None,
        client_kwargs: Optional[dict[str, Any]] = None,
        **kwargs: Any
) -> dict[str, Any]:
    if not base_url:
        if config.url:
            base_url = config.url

    if base_url:
        kwargs["base_url"] = base_url

        if (config.http_auth
            and base_url == config.url
            and (not client_kwargs
                 or "auth" not in client_kwargs)):

            if not client_kwargs:
                client_kwargs = {}
            client_kwargs["auth"] = config.http_auth._as_tuple()

    if client_kwargs:
        kwargs["client_kwargs"] = client_kwargs

    return kwargs



import inspect
import os


def read_resource(filename: str, mode: str = "r") -> str | bytes:
    if not os.path.isabs(filename):
        if not (current_frame := inspect.currentframe()):
            raise NotImplementedError  # pragma: no cover
        if not (caller_frame := current_frame.f_back):
            raise NotImplementedError  # pragma: no cover
        caller_filename = caller_frame.f_code.co_filename
        filename = os.path.join(os.path.dirname(caller_filename), filename)
    with open(filename, mode) as fp:
        result: str | bytes = fp.read()
        return result



import os
import sys


def service_name() -> str:
    service_name = sys.argv[0]
    if service_name != "-m":
        return os.path.basename(service_name)
    for a, b in zip(sys.orig_argv[:-1], sys.orig_argv[1:]):
        if a != "-m":
            continue
        c, _, d = b.partition(".")
        if c != "hive":
            continue
        return "hive-" + d.replace("_", "-")
    raise NotImplementedError


try:
    SERVICE_NAME = service_name().removeprefix("hive-")
except Exception:
    SERVICE_NAME = "unknown-service"



from .argument_parser import HiveArgumentParser as ArgumentParser
from .config import read as read_config
from .datetime import parse_datetime, utc_now
from .resource import read_resource
from .service_name import SERVICE_NAME
from .uuid import parse_uuid
from .xdg import user_cache_dir, user_config_dir

__all__ = [
    "ArgumentParser",
    "parse_datetime",
    "parse_uuid",
    "read_config",
    "read_resource",
    "SERVICE_NAME",
    "user_cache_dir",
    "user_config_dir",
    "utc_now",
]



import os

from typing import Optional, TypeAlias

LogLevel: TypeAlias = int | str


def getenv_log_level(default: Optional[LogLevel] = None) -> Optional[LogLevel]:
    if default is not None:
        default = str(default)

    level = os.environ.get("LL", default)
    if not level:
        return None
    try:
        return int(level)
    except ValueError:
        return level.upper()
